from locust import Locust
from locust.exception import LocustError
from core import RealBrowserClient, ChromeLocustDriver, FirefoxLocustDriver, PhantomJSLocustDriver

class RealBrowserLocust(Locust):
    client = None
    timeout = 30
    screen_width = None
    screen_height = None

    def __init__(self):
        super(RealBrowserLocust, self).__init__()
        if self.screen_width is None:
            raise LocustError("You must specify a screen_width for the browser")
        if self.screen_height is None:
            raise LocustError("You must specify a screen_height for the browser")

class ChromeLocust(RealBrowserLocust):
    """
    This is the abstract Locust class which should be subclassed. It provides a Firefox webdriver that logs GET's and waits to locust
    """
    def __init__(self):
        super(ChromeLocust, self).__init__()
        self.client = RealBrowserClient(ChromeLocustDriver(), self.timeout, self.screen_width, self.screen_height)

class FirefoxLocust(RealBrowserLocust):
    """
    This is the abstract Locust class which should be subclassed. It provides a Firefox webdriver that logs GET's and waits to locust
    """
    def __init__(self):
        super(FirefoxLocust, self).__init__()
        self.client = RealBrowserClient(FirefoxLocustDriver(), self.timeout, self.screen_width, self.screen_height)

class PhantomJSLocust(RealBrowserLocust):
    """
    This is the abstract Locust class which should be subclassed. It provides a PhantomJS webdriver that logs GET's and waits to locust
    """
    def __init__(self):
        super(PhantomJSLocust, self).__init__()
        self.client = RealBrowserClient(PhantomJSLocustDriver(), self.timeout, self.screen_width, self.screen_height)



import time
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait

from locust import Locust, events
from locust.exception import StopLocust


def wrapForLocust(request_type, name, func, *args, **kwargs):
    """
    Internall wrapper function that calls a given func with args and kwargs, and triggers locust events. The request type and name params for
    locusts events.request_success.fire are passed through from the params

    Args:
        request_type (str): the type of request
        name (str): name to be reported to events.request_*.fire
        func (Function): function to be timed and wrapped
        *args: arguments to be used when calling func
        **kwargs: Arbitrary keyword args used for calling func

    Returns:
        func(*args, **kwargs) if this function invocation does not raise an exception

    Raises:
        StopLocust: whenever func raises an exception, this exception is catched, logged to locust as a failure and a StopLocust exception is raised.
    """
    try:
        start_time = time.time()
        result = func(*args, **kwargs)
    except Exception as e:
        total_time = int((time.time() - start_time) * 1000)
        events.request_failure.fire(request_type=request_type, name=name, response_time=total_time, exception=e)
        raise StopLocust()
    else:
        total_time = int((time.time() - start_time) * 1000)
        events.request_success.fire(request_type=request_type, name=name, response_time=total_time, response_length=0)
        return result


class FirefoxLocustDriver(webdriver.Firefox):

    def get(self, url):
        return wrapForLocust("GET", url, super(FirefoxLocustDriver, self).get, url)

class PhantomJSLocustDriver(webdriver.PhantomJS):

    def get(self, url):
        return wrapForLocust("GET", url, super(PhantomJSLocustDriver, self).get, url)

class ChromeLocustDriver(webdriver.Chrome):

    def get(self, url):
        return wrapForLocust("GET", url, super(ChromeLocustDriver, self).get, url)

class RealBrowserClient(object):

    def __init__(self, driver, wait_time_to_finish, screen_width, screen_height):
        self.driver = driver
        self.driver.set_window_size(screen_width, screen_height)
        self.wait = WebDriverWait(self.driver, wait_time_to_finish)

    def waitUntil(self, method, message=""):
        return wrapForLocust("Wait for", message, self.wait.until, method, message)

    def __getattr__(self, attr):
        """Forward all messages this client doesn't understand to it's webdriver"""
        return getattr(self.driver, attr)



from distutils.core import setup
setup(
  name = 'realbrowserlocusts',
  packages = ['realbrowserlocusts'],
  version = '0.2',
  description = 'Minimal set of real browser (Firefox, Chrome and PhantomJS) locusts to be used in conjuntion with locust.io',
  author = 'Nick Bocuart',
  author_email = 'nboucart@gmail.com',
  url = 'https://github.com/nickboucart/realbrowserlocusts',
  download_url = 'https://github.com/nickboucart/realbrowserlocusts/tarball/0.2',
  keywords = ['testing', 'locust'],
  classifiers = [],
)
